#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <Outpost2DLL.h>
#include <OP2Helper.h>
#include "SmartLava.h" //<- needed

char MapName[]			= "on4_02.map";//Flood Plain
char LevelDesc[]		= "Smart Lava Test";
char TechtreeName[]		= "MULTITEK.TXT";
SDescBlock DescBlock	= {MultiLastOneStanding, 6, 12, 1};

//Two arrays are made, one of LavaNodes to eventualy hold the real
//nodes, and one of BasicNodes, to quickly define all the nodes.
//Also the number of nodes are stored in NumNodes
int NumNodes = 11;
LavaNode LavaNodes[11];
BasicNode DefNodes[11] = {
  //Each definiton follows this pattern:
  //BasicNode(LocationOfNode, MaxRadius, NumberOfNeighborIndexes, AnyNeighborIndexes...)
    BasicNode(LOCATION(62+31,72-1), 3, 1, 1),//0 <- indexes
    BasicNode(LOCATION(54+31,76-1), 5, 3, 0, 2, 4),
    BasicNode(LOCATION(42+31,79-1), 4, 2, 1, 3),
    BasicNode(LOCATION(32+31,87-1), 5, 3, 2, 5, 6),
    BasicNode(LOCATION(52+31,62-1), 5, 1, 1),
    BasicNode(LOCATION(27+31,96-1), 3, 1, 3),//5
    BasicNode(LOCATION(37+31,94-1), 3, 2, 3, 7),
    BasicNode(LOCATION(38+31,102-1), 4, 2, 6, 8),
    BasicNode(LOCATION(31+31,108-1), 3, 2, 7, 9),
    BasicNode(LOCATION(24+31,106-1), 4, 2, 8, 10),
    BasicNode(LOCATION(17+31,116-1), 5, 1, 9),//10
};

//Similarly to the nodes, volcanoes are held in an array of VolcanoTrees,
//and the number of volcanoes is kept.
int NumVolcanoes = 1;
VolcanoTree Volcanoes[1];

BOOL APIENTRY DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
	if(fdwReason == DLL_PROCESS_ATTACH)
		DisableThreadLibraryCalls(hinstDLL);
    else if(fdwReason == DLL_PROCESS_DETACH) //<- code added to clean up afterwards
        SmartLavaCleanUp();//MUST be called with the previous live of code

    return TRUE;
}

//Used to help clean up after fired triggers
Trigger Volcano1Trigger;

int InitProc()
{
    //Turn on error finding code
    SetDoShowErrors(true);

    //Send both arrays and their sizes, this function MUST be called
	SmartLavaInit(LavaNodes, NumNodes, Volcanoes, NumVolcanoes);

	//Send map cell details
    SetLavaCells(2, cellSlowPassible1, cellMediumPassible2);
    SetImpassibleCells(5, cellImpassible1, cellImpassible2, cellNorthCliffs,
        cellCliffsHighSide, cellCliffsLowSide);

	//Convert and copy basic nodes, MUST be done after level details are sent
    for(int i = 0; i < NumNodes; i++)
        LavaNodes[i] = LavaNode(&DefNodes[i]);

    //Set time trigger for the volcano
    Volcano1Trigger = CreateTimeTrigger(true, true, 400, "SetVolcano1");//erupts the volcano at mark 4
	return 1;
}

void AIProc()
{
    //MUST be called here
    LavaAIProc();
}

void __cdecl GetSaveRegions(struct BufferDesc &bufDesc)
{
	bufDesc.bufferStart = 0;
	bufDesc.length = 0;
}

int StatusProc()
{
	return 0;
}

SCRIPT_API void NoResponseToTrigger(){}

SCRIPT_API void SetVolcano1()
{
    //Creates the lava flow animation on the side of the volcano
    FlowSW(LOCATION(66+31, 69-1));
    //Destroys the previous trigger
    Volcano1Trigger.Destroy();
    //Sets a time trigger for the next action
    Volcano1Trigger = CreateTimeTrigger(true, true, 100, "TriggerVolcano1");//1 mark
}

SCRIPT_API void TriggerVolcano1()
{
    GameMap::SetLavaPossible(LOCATION(65+31, 71-1), true);
    TethysGame::SetEruption(65+31, 71-1, 100);
    //Destroys the previous trigger
    Volcano1Trigger.Destroy();
    //Sets a time trigger for the next action
    Volcano1Trigger = CreateTimeTrigger(true, true, 1000, "EruptVolcano1");//10 marks
}

SCRIPT_API void EruptVolcano1()
{
    //Destroys the previous trigger
    Volcano1Trigger.Destroy();
    //Stops the lava animation
    FreezeSW(LOCATION(66+31, 69-1));
    //Loads the first volcano, it will spread every 20-25 marks
    LoadVolcano(&Volcanoes[0], -1, &LavaNodes[0], LavaNodes[0].MaxRadius, true, 2000, 2500);
}

